/********************************************************************
 * Appareil Photo de surveillance pour imprimante 3D ou tout autre 
 * usage. Un ESP32_Cam, connecté sur le Wi-Fi, et spécialement 
 * asservi à une micro caméra OVE2640, capture les photos et les 
 * envoient sur le réseau . Une page HTML résidente assure la 
 * page d'accueil et de présentation ... Merci à Ruiz et Sara Santo
 * pour leur aide pour la source HTML
 ********************************************************************/

 //Appel des librairies utilisées 
#include "WiFi.h"
#include "esp_camera.h"
#include "esp_timer.h"
#include "img_converters.h"
#include "Arduino.h"
#include "soc/soc.h"           
#include "soc/rtc_cntl_reg.h"  
#include "driver/rtc_io.h"
#include <ESPAsyncWebServer.h>
#include <StringArray.h>
#include <SPIFFS.h>
#include <FS.h>

//Déclaration de la sortie pour led flash
int ledFlash = 4;

//Saisie des identifiant de la box, reseau et clé 
const char* ssid = "ZZZQQQTTTRRROOO";
const char* password = "Zzzzzzzzzzzzzzzzzzz";

//Créer une entrée asynchrone serveur réseau sur le port 80
AsyncWebServer server(80);

boolean takeNewPhoto = false;

//Nom du fichier photo à enregistrer dans SPIFFS
#define FILE_PHOTO "/photo.jpg"

//Broches du module de caméra OV2640 (CAMERA_MODEL_AI_THINKER)
#define PWDN_GPIO_NUM     32
#define RESET_GPIO_NUM    -1
#define XCLK_GPIO_NUM      0
#define SIOD_GPIO_NUM     26
#define SIOC_GPIO_NUM     27
#define Y9_GPIO_NUM       35
#define Y8_GPIO_NUM       34
#define Y7_GPIO_NUM       39
#define Y6_GPIO_NUM       36
#define Y5_GPIO_NUM       21
#define Y4_GPIO_NUM       19
#define Y3_GPIO_NUM       18
#define Y2_GPIO_NUM        5
#define VSYNC_GPIO_NUM    25
#define HREF_GPIO_NUM     23
#define PCLK_GPIO_NUM     22

//Génere la page html d'accueil, le titre et le fond
const char index_html[] PROGMEM = R"rawliteral(
<!DOCTYPE HTML><html>
<head>
<title>ESP32-CAM</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <style>
    html {font-family: Arial; display: inline-block; text-align: center;}
    p {  font-size: 1.2rem;}
    body {  margin: 0;}
    .topnav { overflow: hidden; background-color: #4B1D3F; color: white; font-size: 1.7rem; }
    .content { padding: 20px; }
    .reading { font-size: 2.8rem; }
    .packet { color: #bebebe; }
  </style>
</head>
<body>
  <div class="topnav">
    <h3>Camera Surveillance 3D Printer</h3>
    <p>Attendre de 5 secondes suite à la prise de Photo.</p>
    <p>
      <button onclick="capturePhoto()">CLIC PHOTO</button>
      <button onclick="location.reload();">AFFICHAGE</button>
    </p>
  </div>
  <div><img src="saved-photo" id="photo" width="70%"></div>
</body>
<script>
  var deg = 0;
  function capturePhoto() {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', "/capture", true);
    xhr.send();
  }
  function isOdd(n) { return Math.abs(n % 2) == 1; }
</script>
</html>)rawliteral";

void setup() {

  //Connexion au Wi-Fi
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
  }
  if (!SPIFFS.begin(true)) {
    ESP.restart();
  }
  else {
    delay(500);
  }


  // Désactiver le « détecteur de baisse de tension »
  WRITE_PERI_REG(RTC_CNTL_BROWN_OUT_REG, 0);

  // Configuration et paramétrage du module Caméra OV2640
  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = Y2_GPIO_NUM;
  config.pin_d1 = Y3_GPIO_NUM;
  config.pin_d2 = Y4_GPIO_NUM;
  config.pin_d3 = Y5_GPIO_NUM;
  config.pin_d4 = Y6_GPIO_NUM;
  config.pin_d5 = Y7_GPIO_NUM;
  config.pin_d6 = Y8_GPIO_NUM;
  config.pin_d7 = Y9_GPIO_NUM;
  config.pin_xclk = XCLK_GPIO_NUM;
  config.pin_pclk = PCLK_GPIO_NUM;
  config.pin_vsync = VSYNC_GPIO_NUM;
  config.pin_href = HREF_GPIO_NUM;
  config.pin_sscb_sda = SIOD_GPIO_NUM;
  config.pin_sscb_scl = SIOC_GPIO_NUM;
  config.pin_pwdn = PWDN_GPIO_NUM;
  config.pin_reset = RESET_GPIO_NUM;
  config.xclk_freq_hz = 20000000;
  config.pixel_format = PIXFORMAT_JPEG;

  if (psramFound()) {
    config.frame_size = FRAMESIZE_UXGA;
    config.jpeg_quality = 10;
    config.fb_count = 2;
  } else {
    config.frame_size = FRAMESIZE_SVGA;
    config.jpeg_quality = 12;
    config.fb_count = 1;
  }
  //Initialisation de la caméra 
  esp_err_t err = esp_camera_init(&config);
  if (err != ESP_OK) {
    ESP.restart();
  }

  //Route pour la racine et la page Web
  server.on("/", HTTP_GET, [](AsyncWebServerRequest * request) {
    request->send_P(200, "text/html", index_html);
  });

  server.on("/capture", HTTP_GET, [](AsyncWebServerRequest * request) {
    takeNewPhoto = true;
    request->send_P(200, "text/plain", "Taking Photo");
  });

  server.on("/saved-photo", HTTP_GET, [](AsyncWebServerRequest * request) {
    request->send(SPIFFS, FILE_PHOTO, "image/jpg", false);
  });

  //Démarage du serveur
  server.begin();

  // DDR de la sortie pour le flash et mise à l'extinction
  pinMode(ledFlash,OUTPUT);
  digitalWrite(ledFlash,LOW);

}

void loop() {
  // Si le bouton "prise de vue" est activé, allumage du flash et capture de photo
  if (takeNewPhoto) {
    digitalWrite(ledFlash,HIGH);
    capturePhotoSaveSpiffs(); //Saut au sous programme de prise de photo
    takeNewPhoto = false;
    digitalWrite(ledFlash,LOW); //Extinction du Flash
  }
  delay(1);
}

//Vérifie si la capture photo a réussi
bool checkPhoto( fs::FS &fs ) {
  File f_pic = fs.open( FILE_PHOTO );
  unsigned int pic_sz = f_pic.size();
  return ( pic_sz > 100 );
}

//Capture une photo et enregistre-la dans SPIFFS
void capturePhotoSaveSpiffs( void ) {
  camera_fb_t * fb = NULL; // pointer
  bool ok = 0; //Indicateur si la photo a été prise correctement

  do {
    //Prend une photo avec l'appareil photo

    fb = esp_camera_fb_get();
    if (!fb) {
      return;
    }

    //Nomme le fichier photo
    File file = SPIFFS.open(FILE_PHOTO, FILE_WRITE);

    //Rentre les données dans le fichier photo
    if (!file) {
    }
    else {
      file.write(fb->buf, fb->len); //Longueur et poids de l'image
    }
    //Fermeture du fichier
    file.close();
    esp_camera_fb_return(fb);

    //Contrôle si le fichier a été correctement sauvé dans SPIFFS
    ok = checkPhoto(SPIFFS);
  } while ( !ok );
}
